@@ -1,21 +1,24 @@
-# ABSTRACT: Simple Stupid Spider base on Web::Scraper inspired by Dancer
+# ABSTRACT: All Powerful Web Harvester, Spider, Scraper fully automated
 
 use strict;
 use warnings;
 
 package Scrappy;
 BEGIN {
-  $Scrappy::VERSION = '0.59';
+  $Scrappy::VERSION = '0.591';
 }
 use FindBin;
 use WWW::Mechanize::Pluggable;
 use File::ShareDir ':ALL';
 use File::Slurp;
 use YAML::Syck;
+use Array::Unique;
+use Try::Tiny;
 
+our $class_History              = [];
 our $class_Instance             = undef;
     $YAML::Syck::ImplicitTyping = 1;
-
+    
 BEGIN {
     use Exporter();
     use vars qw( @ISA %EXPORT_TAGS @EXPORT_OK );
@@ -56,6 +59,11 @@ BEGIN {
         zoom
         proxy
         pause
+        crawl
+        queue
+        cursor
+        denied
+        history
     );
     %EXPORT_TAGS = ( syntax => [ @EXPORT_OK ] );
 }
@@ -74,6 +82,11 @@ sub init {
 }
 
 
+sub reset {
+    return init(@_);
+}
+
+
 sub self {
     die 'No scraper application instance found, please use the `init` method' .
         'before calling any other functions from your package or script.'
@@ -190,6 +203,7 @@ sub form {
 
 sub get {
     my $request = self->get(@_);
+    push @{$class_History}, @_;
     self->{Mech}->{cookie_jar}->scan(\&_cookies_to_session);
     sleep pause();
     return $request;
@@ -530,24 +544,244 @@ sub pause {
     }
 }
 
-1;
+our @_queue = ();
+tie @_queue, 'Array::Unique';
+our $_cursor = 0;
+
+
+sub cursor {
+    $_cursor = $_[0] if $_[0];
+    return $_cursor;
+}
+
+
+sub queue {
+    return @_ ? push @_queue, @_ : @_queue;
+}
+
+
+sub crawl {
+    my ($url, $actions) = @_;
+    
+    $_queue[cursor()] = $url;
+    
+    doPage:
+    
+    try {
+        get $_queue[cursor()];
+    }
+    catch {
+        warn "problem fetching " . $_queue[cursor()];
+        goto nextPage;
+    };
+    
+    try {
+        loaded;
+    }
+    catch {
+        warn "problem loading " . $_queue[cursor()];
+        goto nextPage;
+    };
+    
+    warn "fetching page " . $_queue[cursor()] if $ENV{Scrappy_Trace};
+    
+    # process actions
+    if ("hash" eq lc ref $actions) {
+        while (my($selector, $function) = each(%{$actions})) {
+            my $findings = grab $selector, tattr();
+            foreach (@{$findings}) {
+                $function->(element($_));
+            }
+        }
+    }
+    
+    nextPage:
+    goto doPage if $_queue[++$_cursor];
+}
+
+sub element {
+    my $element = shift;
+    foreach my $attr(keys %{$element}) {
+        {
+            no warnings 'redefine';
+            no strict 'refs';
+            *{"Scrappy::Element::$attr"} = sub {
+                return shift->{$attr};
+            }
+        }
+    }
+    bless $element, 'Scrappy::Element';
+}
+
+sub tattr {
+    return {
+        'abbr'           => '@abbr',
+        'accept-charset' => '@accept',
+        'accept'         => '@accept',
+        'accesskey'      => '@accesskey',
+        'action'         => '@action',
+        'align'          => '@align',
+        'alink'          => '@alink',
+        'alt'            => '@alt',
+        'archive'        => '@archive',
+        'axis'           => '@axis',
+        'background'     => '@background',
+        'bgcolor'        => '@bgcolor',
+        'border'         => '@border',
+        'cellpadding'    => '@cellpadding',
+        'cellspacing'    => '@cellspacing',
+        'char'           => '@char',
+        'charoff'        => '@charoff',
+        'charset'        => '@charset',
+        'checked'        => '@checked',
+        'cite'           => '@cite',
+        'class'          => '@class',
+        'classid'        => '@classid',
+        'clear'          => '@clear',
+        'code'           => '@code',
+        'codebase'       => '@codebase',
+        'codetype'       => '@codetype',
+        'color'          => '@color',
+        'cols'           => '@cols',
+        'colspan'        => '@colspan',
+        'compact'        => '@compact',
+        'content'        => '@content',
+        'coords'         => '@coords',
+        'data'           => '@data',
+        'datetime'       => '@datetime',
+        'declare'        => '@declare',
+        'defer'          => '@defer',
+        'dir'            => '@dir',
+        'disabled'       => '@disabled',
+        'enctype'        => '@enctype',
+        'face'           => '@face',
+        'for'            => '@for',
+        'frame'          => '@frame',
+        'frameborder'    => '@frameborder',
+        'headers'        => '@headers',
+        'height'         => '@height',
+        'href'           => '@href',
+        'hreflang'       => '@hreflang',
+        'hspace'         => '@hspace',
+        'http'           => '@http-equiv',
+        'id'             => '@id',
+        'ismap'          => '@ismap',
+        'label'          => '@label',
+        'lang'           => '@lang',
+        'language'       => '@language',
+        'link'           => '@link',
+        'longdesc'       => '@longdesc',
+        'marginheight'   => '@marginheight',
+        'marginwidth'    => '@marginwidth',
+        'maxlength'      => '@maxlength',
+        'media'          => '@media',
+        'method'         => '@method',
+        'multiple'       => '@multiple',
+        'name'           => '@name',
+        'nohref'         => '@nohref',
+        'noresize'       => '@noresize',
+        'noshade'        => '@noshade',
+        'nowrap'         => '@nowrap',
+        'object'         => '@object',
+        'onblur'         => '@onblur',
+        'onchange'       => '@onchange',
+        'onclick'        => '@onclick',
+        'ondblclick'     => '@ondblclick',
+        'onfocus'        => '@onfocus',
+        'onkeydown'      => '@onkeydown',
+        'onkeypress'     => '@onkeypress',
+        'onkeyup'        => '@onkeyup',
+        'onload'         => '@onload',
+        'onmousedown'    => '@onmousedown',
+        'onmousemove'    => '@onmousemove',
+        'onmouseout'     => '@onmouseout',
+        'onmouseover'    => '@onmouseover',
+        'onmouseup'      => '@onmouseup',
+        'onreset'        => '@onreset',
+        'onselect'       => '@onselect',
+        'onsubmit'       => '@onsubmit',
+        'onunload'       => '@onunload',
+        'profile'        => '@profile',
+        'prompt'         => '@prompt',
+        'readonly'       => '@readonly',
+        'rel'            => '@rel',
+        'rev'            => '@rev',
+        'rows'           => '@rows',
+        'rowspan'        => '@rowspan',
+        'rules'          => '@rules',
+        'scheme'         => '@scheme',
+        'scope'          => '@scope',
+        'scrolling'      => '@scrolling',
+        'selected'       => '@selected',
+        'shape'          => '@shape',
+        'size'           => '@size',
+        'span'           => '@span',
+        'src'            => '@src',
+        'standby'        => '@standby',
+        'start'          => '@start',
+        'style'          => '@style',
+        'summary'        => '@summary',
+        'tabindex'       => '@tabindex',
+        'target'         => '@target',
+        'text'           => '@text',
+        'title'          => '@title',
+        'type'           => '@type',
+        'usemap'         => '@usemap',
+        'valign'         => '@valign',
+        'value'          => '@value',
+        'valuetype'      => '@valuetype',
+        'version'        => '@version',
+        'vlink'          => '@vlink',
+        'vspace'         => '@vspace',
+        'width'          => '@width',
+        'text'           => 'TEXT',
+        'html'           => 'HTML',
+    };
+    # need xml and json support maybe?
+}
+
+
+sub history {
+    return @{$class_History};
+}
+
+
+sub denied {
+    my ($last) = reverse history;
+    return 1 if (page ne $last);
+}
+
+init;
 __END__
 =pod
 
 =head1 NAME
 
-Scrappy - Simple Stupid Spider base on Web::Scraper inspired by Dancer
+Scrappy - All Powerful Web Harvester, Spider, Scraper fully automated
 
 =head1 VERSION
 
-version 0.59
+version 0.591
 
 =head1 SYNOPSIS
 
+Crawl an entire website or more with three lines of code.
+
+    #!/usr/bin/perl
+    use Scrappy qw/:syntax/;
+        
+    crawl 'http://somewebsite.com' {
+        'a' => sub { queue shift->href },
+        '/*' => {
+            # do something
+        }
+    };
+
+Spider, Scrape or Harvest data from websites like never before, with ease.
+
     #!/usr/bin/perl
     use Scrappy qw/:syntax/;
         
-    init;
     user_agent random_ua;
     
     get 'http://search.cpan.org/recent';
@@ -559,6 +793,16 @@ version 0.59
     
     print $_->{name}, "\n" for list var->{modules};
 
+Trace page fetches during crawling by using the 'Scrappy_Trace' environment variable, e.g. ...
+
+    $ENV{Scrappy_Trace} = 1;
+    crawl 'http://somewebsite.com' {
+        'a' => sub { queue shift->href },
+        '/*' => {
+            # do something
+        }
+    };
+
 =head1 DESCRIPTION
 
 Scrappy is an easy (and hopefully fun) way of scraping, spidering, and/or
@@ -583,6 +827,13 @@ instance.
 
     my $scraper = init;
 
+=head2 reset
+
+The reset method is an alias to the init method. This function should be called before
+issuing any other commands as this function creates the application instance all
+other functions will use. This function returns the current scraper application
+instance.
+
 =head2 self
 
 This method returns the current scraper application instance which can also be
@@ -968,9 +1219,69 @@ download one could obviously...
     download $requested_url, '/tmp';
     sleep pause();
 
+=head2 cursor
+
+The cursor method is used internally by the crawl method to determine what
+pages in the queue should be fetched next after the completion of the current
+fetch. This method returns the position of the cursor in the queue.
+
+=head2 queue
+
+The queue method is used to add valid URIs to the page fetching queue used by
+the crawl method internally, or to return the list of added URIs in the order
+received/input.
+
+    queue $new_url;
+    my @urls = queue;
+
+=head2 crawl
+
+The crawl method is designed to automatically and systematically crawl, spider,
+or fetch webpages and perform actions on selected elements on each page.
+
+    crawl $starting_url, {
+        'a' => sub {
+            # find all links and add them to the queue to be crawled
+            queue shift->href;
+        },
+        '/*' => sub {
+            # /* simply matches the root node, same as using 'body' in
+            # html page context, maybe do soemthing with shift->text or shift->html
+        },
+        'img' => sub {
+            # print all image URLs
+            print shift->src, "\n"
+        }
+    };
+
+=head2 history
+
+The history method returns a list of visted pages.
+
+    get $url_a;
+    get $url_b;
+    get $url_c;
+
+    print history;
+
+=head2 denied
+
+The denied method is a simple shortcut to determine if the page you
+requested got loaded or redirected. This method is very useful on systems
+that require authentication and redirect if not authorized. This function
+return boolean, 1 if the current page doesn't match the requested page.
+
+    get $url_to_dashboard;
+    if (denied) {
+        # do login, again
+    }
+    else {
+        # resume ...
+    }
+
 =head1 AUTHOR
 
-Al Newkirk <awncorp@cpan.org>
+  Al Newkirk <awncorp@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
